% compute the path for trace norm regularization with the trace norm
% different types of data and adaptiveness

clear all;
data_type = 'inconsistent';
% data_type = 'consistent';

% gamma = 0 % non adaptive
gamma = 1/2; % adaptive
% gamma = 1; % adaptive, alternative

% select data type
switch data_type
    case 'inconsistent'
        seed=31;
        %  for seed=1:100
        randn('state',seed);
        rand('state',seed);

        dx = 4;
        dy = 4;
        n = 1000;
        r = 2;
        noise_std = .125;


    case 'consistent'
        seed=22;
        %  for seed=1:100
        randn('state',seed);
        rand('state',seed);

        dx = 4;
        dy = 4;
        n = 1000;
        r = 2;
        noise_std = .5;
end


% generate random Gaussians and W0
Sigma_xxG = randn(dx,dx);
Sigma_yyG = randn(dy,dy);
Sigma_xx = Sigma_xxG' * Sigma_xxG;
Sigma_yy = Sigma_yyG' * Sigma_yyG;
Sigma_mm = kron( Sigma_yy, Sigma_xx);
W0 = randn(dx,dy);
[U,S,V] = svd(W0);
S(r+1:end,r+1:end) = 0;
W0 = U * S * V';
U0 = U(:,1:r); U0orth = U(:,r+1:end);
V0 = V(:,1:r); V0orth = V(:,r+1:end);

X = randn(n,dx) * Sigma_xxG;
Y = randn(n,dy) * Sigma_yyG;
 Z = zeros(n,1);
        noise = randn(n,1) * noise_std;
        for i=1:n
            Z(i) =  X(i,:) * W0 * Y(i,:)' + noise(i);
        end
        Sigma = zeros(dx*dy,dx*dy);
Q = zeros(dx,dy);
for i=1:n
    Sigma = Sigma + kron(Y(i,:)',X(i,:)') * kron(Y(i,:)',X(i,:)')';
    Q = Q + X(i,:)' * Y(i,:) * Z(i);
end
Sigma = Sigma / n;
Q = Q / n;

% compute least-square estimate
W_LS = reshape( ( Sigma + 1e-10 * eye(dx*dy) ) \ Q(:), dx ,dy );

% adaptive (if required)
if gamma>0
    % compute least-square estimate
    [U_LS,s_LS,V_LS] = svd(W_LS);
    A_LS = U_LS * diag( [ diag(s_LS + n^(-1/2) ).^(-gamma) ; n^(-gamma/2) * ones( dx-min(dx,dy) ,1 ) ] ) * U_LS';
    B_LS = V_LS * diag( [ diag(s_LS + n^(-1/2)).^(-gamma) ; n^(-gamma/2) * ones( dy-min(dx,dy) ,1 ) ] ) * V_LS';
    A_LS_inv = U_LS * diag( [ diag(s_LS+ n^(-1/2)).^(gamma) ; n^(gamma/2) * ones( dx-min(dx,dy) ,1 ) ] ) * U_LS';
    B_LS_inv = V_LS * diag( [ diag(s_LS+ n^(-1/2)).^(gamma) ; n^(gamma/2) * ones( dy-min(dx,dy) ,1 ) ] ) * V_LS';

    Sigma_f = kron( B_LS_inv , A_LS_inv ) * Sigma * kron ( B_LS_inv , A_LS_inv )';
    Q_f = A_LS_inv * Q * B_LS_inv;
    W_LS_f = reshape( ( Sigma_f + 1e-10 * eye(dx*dy) ) \ Q_f(:), dx ,dy );
else
    Sigma_f = Sigma;
    Q_f = Q;
    W_LS_f = W_LS;

end

% check condition
vectId = eye(r); vectId = vectId(:);
Lambda = ( kron(V0orth,U0orth)' * inv(Sigma_mm) *  kron(V0orth,U0orth) ) \ ...
    ( kron(V0orth,U0orth)' * inv(Sigma_mm) * kron(V0,U0) * vectId );
Lambda = reshape(Lambda, dx-r,dy-r);
LambdaEIG = max( svd(Lambda) )
if (LambdaEIG < 1), disp('consistency condition satisfied!');
else
    disp('consistency condition not satisfied!');
end


% compute start and end of path, with sampling grid
alpha = 1e-4;   % minimum barrier parameter
lambdamax = max(svd(Q_f)) * 2;
lambdamin = 1e-8 * sum( Q_f(:) .* W_LS_f(:) ) / sum( svd( W_LS_f ) );
nlambdas = 200;
lambdas = exp( log(lambdamax) + ( log(lambdamin) - log(lambdamax) ) / nlambdas * ( 0:(nlambdas-1) ));

optparam.tol = 1e-16;   % NB : convergence up to strong precision
optparam.kmax = 100;
optparam.tol_df = 1e-16;
optparam.display= 0;
w = zeros(dx*dy,1);
svds = [];

fprintf('Trying %d values of lambds\n',length(lambdas));
for ilambda = 1:length(lambdas)
    lambda = lambdas(ilambda);
    fprintf('%d - lambda = %1.2e\n', ilambda, lambda);
    barriers = 10.^[0:1:4];
    for ibar = 1:length(barriers)
        [w,exit_parameters] = minimize_newton(w,@tracelasso,optparam,Sigma_f,Q_f,lambda,alpha/barriers(ibar));
    end
    if gamma==0
        W = reshape(w,dx,dy);
    else
        W =  A_LS_inv * reshape(w,dx,dy) * B_LS_inv ;
    end
    svds(:,ilambda) = svd(W);
    dists(ilambda) = norm(W-W0,'fro')^2;

end



plot(-log(lambdas),svds','linewidth',2); hold on;
plot(-log(lambdas),repmat(svd(W0),1,length(lambdas))',':','linewidth',2); hold off
axis( [ min(-log(lambdas) ) max(-log(lambdas) ) 0 max(svds(:))*1.1] );
xlabel('-log(\lambda)','fontsize',16);
ylabel('singular values','fontsize',16);



