% compute the path for trace norm regularization with the trace norm
% try different types of data (inconsistent of consistent)

clear all;

nrep = 100;

for seed=1:nrep
    randn('state',seed);
    rand('state',seed);
    seed
    dx = 4;
    dy = 4;
    n = 10000;
    r = 2;
    noise_std = .25;

    % generate random Gaussians
    Sigma_xxG = randn(dx,dx);
    Sigma_yyG = randn(dy,dy);
    Sigma_xx = Sigma_xxG' * Sigma_xxG;
    Sigma_yy = Sigma_yyG' * Sigma_yyG;
    Sigma_mm = kron( Sigma_yy, Sigma_xx);
    W0 = randn(dx,dy);
    [U,S,V] = svd(W0);
    [a,b]=sort(diag(S),'descend');
    b = [ b; (min(size(W0))+1:max(size(W0)))' ];
    S=S(b(1:size(S,1)),b(1:size(S,2))); U=U(:,(1:size(S,1))); V=V(:,b(1:size(S,2)));

    S(r+1:end,r+1:end) = 0;
    S = S / sum(sum(S)); % normalize to unit trace norm
    W0 = U * S * V';
    U0 = U(:,1:r); U0orth = U(:,r+1:end);
    V0 = V(:,1:r); V0orth = V(:,r+1:end);

    X = randn(n,dx) * Sigma_xxG;
    Y = randn(n,dy) * Sigma_yyG;
    Z = diag( X * W0 * Y' )  + randn(n,1) * noise_std;
    Sigma = zeros(dx*dy,dx*dy);
    Q = zeros(dx,dy);
    for i=1:n
        Sigma = Sigma + kron(Y(i,:)',X(i,:)') * kron(Y(i,:)',X(i,:)')';
        Q = Q + X(i,:)' * Y(i,:) * Z(i);
    end
    Sigma = Sigma / n;
    Q = Q / n;

    % compute least-square estimate
    W_LS = reshape( Sigma \ Q(:), dx ,dy );


    % check condition
    vectId = eye(r); vectId = vectId(:);
    Lambda = ( kron(V0orth,U0orth)' * inv(Sigma_mm) *  kron(V0orth,U0orth) ) \ ...
        ( kron(V0orth,U0orth)' * inv(Sigma_mm) * kron(V0,U0) * vectId );
    Lambda = reshape(Lambda, dx-r,dy-r)
    conditions(seed) = max(svd(Lambda))


    alpha = 1e-4;
    lambdamax = max(svd(Q)) * 2;
    lambdamin = 1e-8 * sum( Q(:) .* W_LS(:) ) / sum( svd( W_LS ) );
    nlambdas = 200;
    lambdas = exp( log(lambdamax) + ( log(lambdamin) - log(lambdamax) ) / nlambdas * ( 0:(nlambdas-1) ));

    optparam.tol = 1e-16;
    optparam.kmax = 100;
    optparam.tol_df = 1e-20;
    optparam.display= 0;
    w = zeros(dx*dy,1);
    svds = [];
    for ilambda = 1:length(lambdas)
        lambda = lambdas(ilambda);
        fprintf('%d - lambda = %1.2e\n', ilambda, lambda);
        barriers = 10.^[0:.5:4];
        for ibar = 1:length(barriers)
            [w,exit_parameters] = minimize_newton(w,@tracelasso,optparam,Sigma,Q,lambda,alpha/barriers(ibar));
        end

        W = reshape(w,dx,dy);
        [U,S,V] = svd(W);
        [a,b]=sort(diag(S),'descend');
        b = [ b; (min(size(W))+1:max(size(W)))' ];
        S=S(b(1:size(S,1)),b(1:size(S,2))); U=U(:,(1:size(S,1))); V=V(:,b(1:size(S,2)));

        svds(:,ilambda) = diag(S);
        dists(ilambda) = norm(W-W0,'fro')^2;
        rs5(ilambda) = sum(diag(S)>1e-5);


    end

    ranks  = sum(svds>1e-5);
    mindist5(seed) = min(dists(find(ranks==r)));

end

plot(log10(conditions),(mindist5),'*','markersize',10)
xlabel('log_{10}(|| \Lambda ||_2 )','fontsize',24);
ylabel('minimum RMS','fontsize',24);



