function [f,grad,hess,lambda2, descent] = tracelasso(w,Sigma,Q,lambda,alpha);
[dx dy ] = size(Q);
W = reshape(w,dx,dy);
f = .5 * w' * Sigma * w - sum( w .* Q(:));
[U,S,V] = svd(W,'econ');
s = diag(S);

f = f + lambda * sum( s + 2 * alpha * log( 1 + exp( -s / alpha ) ) );

if nargout>1
    % compute gradient
    grad = Sigma * w - Q(:);

    grad = grad - lambda * reshape( U * diag( ( exp(-s/alpha) - 1 ) ./ ( exp(-s/alpha) + 1 ) ) * V', dx*dy, 1 );

end

if nargout > 2
    % compute hessian
    hess = Sigma;

    % complete svd decomposition
    s = [ s; zeros(max(dx,dy)-length(s),1) ];
    V = [V , null(V')];
    U = [U , null(U')];

    for i=1:dx

        for j=1:dy
            if abs( s(i)-s(j) )> 1e-12
                hess = hess + lambda * 2 / ( 1 + exp( - s(i)/alpha ) ) / ( 1 + exp( - s(j)/alpha ) ) ...
                    * ( exp( - s(j) / alpha ) - exp( - s(i) / alpha )  ) / ( s(i) - s(j) ) ...
                    * kron( V(:,j) * V(:,j)' , U(:,i) * U(:,i)' );
            else
                hess = hess + lambda * 2 / alpha / ( 1 + exp( - s(i)/alpha ) ).^2 ...
                    * exp( - s(i) / alpha ) ...
                    * kron( V(:,j) * V(:,j)' , U(:,i) * U(:,i)' );

            end
        end
    end

    descent = - hess \ grad;
    lambda2 = - grad' * descent;
end